#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import json
import shutil
from functools import wraps
from flask import make_response, request, session, redirect
from flask import url_for, abort, render_template, jsonify, flash
from werkzeug import secure_filename
from eop import app
from eop import cas
from eop.models.forms import DocDistribuerForm, DocRamasserForm, DocRendreForm, ChangementMDPForm

import xmlrpclib

class EOPError(Exception): pass

def get_proxy():
    """Appel du service xmlrpc contenu dans /usr/share/eole/controlevnc/controle_vnc_serveur.py
    """
    return xmlrpclib.ServerProxy(app.config["CONTROLEVNC_URL"])

def get_membres(groupe=None, filtrersur=None):
    """Récupérer les membres d'un groupe avec possibilité de filtrer sur un type de membre
    """
    proxy = get_proxy()
    if filtrersur is None :
        lesMembres = proxy.remote_membres(groupe)
    else:
        lesMembres = proxy.remote_membres(groupe, filtrersur)
    return lesMembres

def envoi_mail(serveur, username, groupe, sujetmail, corpsmail):
    """Envoi de mail
    """
    from eop.mail import send_mail

    proxy = get_proxy()
    lesmembres = get_membres(groupe, "eleves")
    user_name = proxy.remote_get_nom(app.config['SECRET_KEY'], username)
    user_mail = proxy.remote_get_mail(app.config['SECRET_KEY'], username)

    de = "%s <%s>" % (user_name, user_mail)
    pour = []
    for m in lesmembres:
        pour.append(proxy.remote_get_mail(app.config['SECRET_KEY'], m))

    send_mail(smtp_server=serveur, mailFrom=de, mailTo=pour, mailSubject=sujetmail.encode('utf8'), text=corpsmail.encode('utf8'))

def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if not session.has_key("username"):
            return redirect(url_for('login', next=cas.url_https(request.url),_external=True))
        else:
            nomfich = os.path.join(app.config["UPLOAD_FOLDER"], ".sessions", session['netid'])
            if not os.path.isfile(nomfich):
                # Cas d'une déconnexion générale de CAS (hors EOP)
                user = cas.CASClient()
                return user.logout()
        return f(*args, **kwargs)
    return decorated_function

class InvalidSecretKeyFile(Exception): pass

def get_username():
    """Vérifie la présence du fichier de session et renvoie le username de la session
       S'il n'existe pas c'est que la session est perdue
    """
    if not session.has_key("netid"):
        return ""
    nomfich = os.path.join(app.config["UPLOAD_FOLDER"], ".sessions", session['netid'])
    if not os.path.isfile(nomfich):
        del(session['netid'])
        del(session['username'])

    if session.has_key("username"):
        return session["username"]
    else:
        return ""

@app.route("/login", methods=["GET", "POST"])
def login():
    """Redirection vers le login CAS
    """
    if request.method == 'POST' and request.form.has_key('logoutRequest'):
        # Requete POST lancée par le SSO lors du logout
        id_logout = cas.parse_tag(request.form['logoutRequest'], "samlp:SessionIndex")
        nomfich = os.path.join(app.config["UPLOAD_FOLDER"], ".sessions", id_logout)
        if os.path.isfile(nomfich): os.remove(nomfich)
        return ""
    else:
        user = cas.CASClient()
        return user.authenticate()

@app.route("/logout")
@login_required
def logout():
    """Déconnexion CAS
    """
    user = cas.CASClient()
    return user.logout()

### GET

@app.route("/documents/rest/groupes", methods = ['GET'])
def get_groups():
    """Récupérer les groupes
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesClasses = proxy.remote_groupes("classe")
    lesNiveaux = proxy.remote_groupes("niveau")
    lesMatieres = proxy.remote_groupes("matiere")
    lesEquipes = proxy.remote_groupes("equipe")
    lesGroupes = proxy.remote_groupes("groupe")
    ret = jsonify({"classes":lesClasses, "matieres":lesMatieres, "niveaux":lesNiveaux, "equipes":lesEquipes, "groupes":lesGroupes})

    return make_response(ret)

@app.route("/documents/rest/devoirs", methods = ['GET'])
def get_devoirs():
    """Récupérer tous les devoirs d'un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
    return make_response(jsonify(lesDevoirs))

@app.route("/documents/rest/secureref/<ref>", methods = ['GET'])
def get_secureref(ref=None):
    """Récupérer la version sécurisée d'une chaine de caractère
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    return make_response(jsonify({"secureref":secure_filename(ref)}))

@app.route("/documents/rest/devoirs/aramasser", methods = ['GET'])
def get_devoirs_a_ramasser():
    """Récupérer les devoirs à ramasser d'un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)

    if type(lesDevoirs) == dict:
        for key in lesDevoirs.keys():
            if lesDevoirs[key]["state"] != "dist":
                lesDevoirs.pop(key)
    return make_response(jsonify(lesDevoirs))

@app.route("/documents/rest/devoirs/arendre", methods = ['GET'])
def get_devoirs_a_rendre():
    """Récupérer les devoirs à rendre d'un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)

    if type(lesDevoirs) == dict:
        for key in lesDevoirs.keys():
            if lesDevoirs[key]["state"] != "ram":
                lesDevoirs.pop(key)
    return make_response(jsonify(lesDevoirs))

@app.route("/documents/rest/devoirs/asupprimer", methods = ['GET'])
def get_devoirs_a_supprimer():
    """Récupérer les devoirs à supprimer d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        proxy = get_proxy()
        lesDevoirs = proxy.remote_get_dev_todel(app.config['SECRET_KEY'], username)
        return make_response(jsonify({"DevoirsASupprimer":lesDevoirs}))
    except Exception, e:
        return make_response(jsonify({"erreur":str(e)}))

@app.route('/documents/rest/listedoctemp/<typedoc>', methods=['GET'])
def get_liste_doc_temp(typedoc=None):
    """Liste les fichiers en attente pour un prof selon le type de document (devoir ou data)
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        username = secure_filename(username).lower()  # le dossier a été créé dans upload_doc avec cette option donc on le recherhce avec la même option

        if typedoc == "devoir":
            destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
        elif typedoc == "data":
            destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
        else:
            destdir = ""

        if not os.path.isdir(destdir):
            return make_response(jsonify({"listedoc":None}))
        else:
            return make_response(jsonify({"listedoc":sorted(os.listdir(destdir))}))
    except Exception, e:
        return make_response(jsonify({"listedoc":None, "erreur": str(e)}))

@app.route("/documents/rest/checkdevoir/<devname>", methods = ['GET'])
def check_devoir(devname=None):
    """Vérifier l'existence d'un nom de devoir pour un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
    if type(lesDevoirs) == dict:
        retour = devname in lesDevoirs.keys()
    else:
        retour = False
    return make_response(jsonify({"checkdevoir":retour}))

#### POST

@app.route('/documents/uploaddoctemp/<typedoc>', methods=['GET', 'POST'])
def upload_doc_temp(typedoc=None):
    """Upload de fichiers pour un prof selon le type de document (devoir ou data)

        Les fichiers uploadés arrivent un par un ici :
        request.files = ImmutableMultiDict([('file', <FileStorage: u'Antarctique4.jpg' ('image/jpeg')>)]
    """
    if request.method == "POST":
        try:
            username = get_username()
            if username == "":
                return redirect(url_for("login"))

            username = secure_filename(username).lower()

            if typedoc == "devoir":
                destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
            elif typedoc == "data":
                destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
            else:
                raise EOPError('upload_doc_temp : typedoc doit être soit "devoir" soit "data"')

            if not os.path.isdir(destdir):
                os.makedirs(destdir)

            for fich in request.files.values():
                ret = "KO"
                if fich:
                    filename = secure_filename(fich.filename).lower()
                    ret = fich.save(os.path.join(destdir, filename))
                    if ret is None: ret = "OK"

            return make_response(jsonify({"uploaddoctemp": ret}))
        except Exception, e:
            return make_response(jsonify({"erreur":str(e)}))
    else:
        return abort(404)

@app.route('/documents/supprimerdoctemp/<typedoc>/<fichier>', methods=['GET', 'POST'])
def supprimer_doc_temp(typedoc=None, fichier=None):
    """Suppression d'un fichier en attente pour un prof selon le type de document (devoir ou data)
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        username = secure_filename(username).lower()

        if fichier is None:
            raise EOPError("supprimer_doc_temp : Pas de nom de fichier")
        else:
            fichier = secure_filename(fichier).lower()

        if typedoc == "devoir":
            fichier_complet = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username, fichier)
        elif typedoc == "data":
            fichier_complet = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username, fichier)
        else:
            fichier = ""

        ret = "La suppression de %s a échoué." % fichier
        if os.path.isfile(fichier_complet):
            ret = os.remove(fichier_complet)
            if ret is None: ret = "Le fichier %s a été supprimé." % fichier

        return make_response(jsonify({"suppression":ret}))
    except Exception, e:
        return make_response(jsonify({"suppression":str(e)}))

@app.route('/documents/viderdoctemp/<typedoc>', methods=['GET', 'POST'])
def vider_doc_temp(typedoc=None):
    """Vidage du dossier temporaire pour un prof selon le type de document (devoir ou data)
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        username = secure_filename(username).lower()

        if typedoc == "devoir":
            dossier = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
        elif typedoc == "data":
            dossier = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
        else:
            dossier = ""

        if not os.path.isdir(dossier):
            return

        fichiers = os.listdir(dossier)

        for f in fichiers:
            chemin = os.path.join(dossier, f)
            if os.path.isfile(chemin):
                os.remove(chemin)

        ret = ("Le dossier a été vidé." if os.listdir(dossier)==[] else "Le dossier n'a pas pu être vidé.")

        return make_response(jsonify({"vidage":ret}))
    except Exception, e:
        return make_response(jsonify({"vidage":str(e)}))

#//
#//@app.route("/documents/rest/devoirs/distribuer/<devname>/<groupname>/<eleve_only>/<in_perso>/<sendmail>/<sujetmail>/<corpsmail>", methods = ['GET','POST'])
#//def devoirs_distribuer(devname=None, groupname=None, eleve_only=None, in_perso=None, sendmail=None, sujetmail=None, corpsmail=None):
    #//"""Distribuer un devoir d'un prof
    #//"""
    #//try:
        #//username = get_username()
        #//if username == "":
            #//return make_response(jsonify({"reload":"Session perdue."}))
#//
        #//if devname is None:
            #//raise EOPError("Aucune référence fournie.")
        #//elif groupname is None:
            #//raise EOPError("Aucun groupe fourni.")
        #//elif eleve_only is None:
            #//raise EOPError("Aucune informations sur les destinataires fournies (élèves seuls / tous).")
        #//elif in_perso is None:
            #//raise EOPError("Aucun informations sur la distribution du devoir (immédiat / plus tard).")
        #//elif sendmail == "true" and sujetmail is None:
            #//raise EOPError("Aucun sujet pour le mail.")
        #//elif sendmail == "true" and corpsmail is None:
            #//raise EOPError("Aucun message pour le mail.")
        #//else:
            #//# Vérification de l'existence de la référence de devoir
            #//username = secure_filename(username).lower()
            #//proxy = get_proxy()
            #//lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
            #//devname = secure_filename(devname).lower()
            #//if type(lesDevoirs) == dict:
                #//if devname in lesDevoirs.keys(): raise EOPError("Cette référence existe déjà.")
#//
            #//# Conversion de "true" ou "false" en True ou False)
            #//eleve_only = (True if eleve_only == "true" else False)
            #//in_perso = (True if in_perso == "true" else False)
#//
            #//# Déplacement des fichiers du devoir vers /home/workgroups/devoirs/<username>/<devname>
            #//# en envoyant chaque fichiers un par un au service de copie xmlrpc
            #//dirtempdev = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
            #//dirtempdata = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
#//
            #//listefich_dirtempdev = os.listdir(dirtempdev)
            #//if listefich_dirtempdev == []:
                #//raise EOPError("Aucun document fourni.")    # il doit y avoir au moins un fichier de devoir
#//
            #//# Traitement du dossier des devoirs
            #//fichiers = os.listdir(dirtempdev)
            #//for un_fichier in fichiers:
                #//chemin = os.path.join(dirtempdev, un_fichier)
                #//if os.path.isfile(chemin):
                    #//with open(chemin, "rb") as handle:
                        #//donnees_fichier = xmlrpclib.Binary(handle.read())
                    #//proxy.remote_copie_fichier(app.config['SECRET_KEY'], username, devname, "devoir", un_fichier, donnees_fichier)
                    #//os.remove(chemin)
#//
            #//# Traitement du dossier des data
            #//if os.path.isdir(dirtempdata):
                #//fichiers = os.listdir(dirtempdata)
                #//for un_fichier in fichiers:
                    #//chemin = os.path.join(dirtempdata, un_fichier)
                    #//if os.path.isfile(chemin):
                        #//with open(chemin, "rb") as handle:
                            #//donnees_fichier = xmlrpclib.Binary(handle.read())
                        #//proxy.remote_copie_fichier(app.config['SECRET_KEY'], username, devname, "data", un_fichier, donnees_fichier)
                        #//os.remove(chemin)
#//
            #//# Appel à l'API de gestion de devoir
            #//# reussi = nombre de personnes auquel le devoir a été distribué
            #//# rate = liste des personnes n'ayant pas reçu le devoir
            #//# dev_name = nom du devoir
            #//reussi, rateu, dev_name = proxy.remote_distribute(app.config['SECRET_KEY'], username, groupname, devname, eleve_only, in_perso)
#//
            #//if reussi > 0 and sendmail == "true":
                #//envoi_mail(app.config["SERVER_SMTP"], username, groupname, sujetmail, corpsmail)
#//
            #//return make_response(jsonify({"distribuer": {"devoir": dev_name, "réussi": reussi, "rate": rateu}}))
#//
    #//except Exception, e:
        #//return make_response(jsonify({"erreur":str(e)}))

@app.route("/documents/rest/devoirs/ramasser/<devname>", methods = ['GET','POST'])
def devoirs_ramasser(devname=None):
    """Ramasser un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        if devname is None:
            raise EOPError("Aucune référence fournie.")
        else:
            proxy = get_proxy()
            reussi, rateu = proxy.remote_ramasse(app.config['SECRET_KEY'], username, devname)

        if rateu:
            raise EOPError("Un problème est survenu lors du ramassage.")
        else:
            ramasserDevoir = u"'%s' a bien été ramassé." % devname
    except Exception, e:
        return make_response(jsonify({"erreur":str(e)}))
    return make_response(jsonify({"ramasser": ramasserDevoir}))

@app.route("/documents/rest/devoirs/rendre", methods = ['POST'])
def devoirs_rendre(devname=None):
    """Rendre un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        # request.form = ImmutableMultiDict([('nomdev', u'devoir'), ('groupe', u'c32'), ('envoimail', u'true'), ('sujetmail', u'sujet'), ('corpsmail', u'contenu')])
        devname = request.form["nomdev"] or None
        groupname = request.form["groupe"] or None
        sujetmail = request.form["sujetmail"] or None
        corpsmail = request.form["corpsmail"] or None
        # Conversion de "true" ou "false" en True ou False)
        sendmail = (True if request.form["envoimail"] == "true" else False)

        if devname is None:
            raise EOPError("Aucune référence fournie.")
        elif sendmail and sujetmail is None:
            raise EOPError("Aucun sujet pour le mail.")
        elif sendmail and corpsmail is None:
            raise EOPError("Aucun message pour le mail.")
        else:
            proxy = get_proxy()
            reussi, rateu = proxy.remote_rendre(app.config['SECRET_KEY'], username, devname)

            if reussi > 0 and sendmail:
                envoi_mail(app.config["SERVER_SMTP"], username, groupname, sujetmail, corpsmail)

        if rateu:
            raise EOPError("Un problème est survenu lors du rendu.")
        else:
            rendreDevoir = u"'%s' a bien été rendu." % devname
    except Exception, e:
        return make_response(jsonify({"erreur":str(e)}))
    return make_response(jsonify({"rendre": rendreDevoir}))

@app.route("/documents/rest/devoirs/supprimer/<devname>", methods = ['GET','POST'])
def devoirs_supprimer(devname=None):
    """Supprimer un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        if devname is None:
            raise EOPError("Aucune référence fournie.")
        else:
            proxy = get_proxy()
            supprimerDevoir = proxy.remote_supprimer(app.config['SECRET_KEY'], username, devname)
            if supprimerDevoir:
                ret = u"Les données de '%s' ont été supprimées." % devname
            else:
                ret = u"Les données de '%s' n'ont pas pu être supprimées." % devname
    except Exception, e:
        return make_response(jsonify({"erreur":str(e)}))
    return make_response(jsonify({"supprimer": ret}))

### ERREURS

@app.errorhandler(404)
def page_erreur(error):
    return render_template('erreurs/404.html'), error.code

@app.route("/testmail", methods = ['GET','POST'])
@login_required
def test_mail():
    """teste envoi de mail
    """
    try:
        envoi_mail(app.config["SERVER_SMTP"], "admin", "c31", "coucou", "salut !\nC'est trop super\nA++")
    except Exception, e:
        return make_response(jsonify({"erreur":str(e)}))
    return make_response(jsonify({"mail": "ok"}))
#//
#//@app.route("/getnom", methods = ['GET','POST'])
#//@login_required
#//def get_nom():
    #//username = get_username()
    #//if username == "":
        #//return make_response(jsonify({"reload":"Session perdue."}))
#//
    #//proxy = get_proxy()
    #//return proxy.remote_get_nom(app.config['SECRET_KEY'], username)
#//
#//@app.route("/getmail", methods = ['GET','POST'])
#//@login_required
#//def get_mail():
    #//username = get_username()
    #//if username == "":
        #//return make_response(jsonify({"reload":"Session perdue."}))
#//
    #//proxy = get_proxy()
    #//return proxy.remote_get_mail(app.config['SECRET_KEY'], username)
#//
#//@app.route("/checkpass/<password>/<int:nbclasses>", methods = ['GET','POST'])
#//@login_required
#//def check_pass(password=None, nbclasses=None):
    #//proxy = get_proxy()
    #//return str(proxy.remote_check_nb_min_classes(app.config['SECRET_KEY'], password, nbclasses))
#//
#//@app.route("/chgt_mdp/<old_mdp>/<new_mdp>", methods = ['GET','POST'])
#//@login_required
#//def chgt_mdp(old_mdp=None, new_mdp=None):
    #//username = get_username()
    #//if username == "":
        #//return make_response(jsonify({"reload":"Session perdue."}))
#//
    #//proxy = get_proxy()
    #//return str(proxy.remote_change_mdp(app.config['SECRET_KEY'], username, old_mdp, new_mdp))

### VUES

#//@app.route("/", methods = ['GET','POST'])
#//@login_required
#//def index():
    #//"""Affichage de la page d'index d'EOP
    #//"""
    #//username = get_username()
    #//proxy = get_proxy()
    #//est_prof = proxy.remote_est_prof(app.config['SECRET_KEY'], username)
#//
    #//if not est_prof:
        #//return render_template('erreurs/non_prof.html', username=username)
    #//else:
        #//# prefixe sera ajouté devant chaque lien
        #//return render_template('index.html', username=username, prefixe="")

@app.route("/", methods = ['GET'])
@login_required
def index():
    return redirect(url_for("documents", _external=True))

@app.route("/documents", methods = ['GET'])
@login_required
def documents():
    """Affichage de la page de gestion de devoir
    """
    username = get_username()
    proxy = get_proxy()
    est_prof = proxy.remote_est_prof(app.config['SECRET_KEY'], username)

    if not est_prof:
        return render_template('erreurs/non_prof.html', username=username)
    else:
        form = DocDistribuerForm(request.form)
        form4 = DocRamasserForm(request.form)
        form5 = DocRendreForm(request.form)
        return render_template('documents.html', username=username, form=form, form4=form4, form5=form5)

@app.route("/documents", methods = ['POST'])
def devoirs_distribuer():
    """Distribuer un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        proxy = get_proxy()
        est_prof = proxy.remote_est_prof(app.config['SECRET_KEY'], username)
        if not est_prof:
            return render_template('erreurs/non_prof.html', username=username)
        else:
            # request.form = ImmutableMultiDict([('groupes', u'["c31","c33"]'), ('envoimail', u'true'), ('eleve_only', u'true'), ('nomdev', u'devoir'), ('sujetmail', u'sujet'), ('in_perso', u'true'), ('corpsmail', u'contenu')])
            devname = request.form["nomdev"] or None
            groupes = request.form["groupes"] or None
            sujetmail = request.form["sujetmail"] or None
            corpsmail = request.form["corpsmail"] or None
            # Conversion de "true" ou "false" en True ou False)
            eleve_only = (True if request.form["eleve_only"] == "true" else False)
            in_perso = (True if request.form["in_perso"] == "true" else False)
            sendmail = (True if request.form["envoimail"] == "true" else False)

            if devname is None:
                raise EOPError("Aucune référence fournie.")
            elif (groupes is None):
                raise EOPError("Aucun groupe fourni.")
            elif eleve_only is None:
                raise EOPError("Aucune informations sur les destinataires fournies (élèves seuls / tous).")
            elif in_perso is None:
                raise EOPError("Aucun informations sur la distribution du devoir (immédiat / plus tard).")
            elif sendmail and sujetmail is None:
                raise EOPError("Aucun sujet pour le mail.")
            elif sendmail and corpsmail is None:
                raise EOPError("Aucun message pour le mail.")
            else:
                # Récupération de la liste des groupes
                groupelist = set(json.loads(groupes))   # transformation de la str en liste puis en set pour enlever les doublons
                if len(groupelist) == 0:
                    raise EOPError("Aucun groupe fourni.")
                username = secure_filename(username).lower()
                proxy = get_proxy()
                lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
                dirtempdev = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
                dirtempdata = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)

                listefich_dirtempdev = os.listdir(dirtempdev)
                if listefich_dirtempdev == []:
                    raise EOPError("Aucun document fourni.")    # il doit y avoir au moins un fichier de devoir

                fichiers_dev = os.listdir(dirtempdev)   # commun à tous les groupes
                retours = {}

                for groupname in groupelist:
                    # Vérification de l'existence de la référence de devoir
                    devname_groupe = secure_filename(devname+"-"+groupname).lower()    # La référence du devoir est suffixée avec le nom groupe
                    if type(lesDevoirs) == dict:
                        if devname_groupe in lesDevoirs.keys(): raise EOPError("Cette référence existe déjà.")

                    # Déplacement des fichiers du devoir vers /home/workgroups/devoirs/<username>/<devname_groupe>
                    # Traitement du dossier des devoirs en envoyant chaque fichiers un par un au service de copie xmlrpc
                    for un_fichier in fichiers_dev:
                        chemin = os.path.join(dirtempdev, un_fichier)
                        if os.path.isfile(chemin):
                            with open(chemin, "rb") as handle:
                                donnees_fichier = xmlrpclib.Binary(handle.read())
                            proxy.remote_copie_fichier(app.config['SECRET_KEY'], username, devname_groupe, "devoir", un_fichier, donnees_fichier)

                    # Traitement du dossier des data en envoyant chaque fichiers un par un au service de copie xmlrpc
                    if os.path.isdir(dirtempdata):
                        fichiers_data = os.listdir(dirtempdata)
                        for un_fichier in fichiers_data:
                            chemin = os.path.join(dirtempdata, un_fichier)
                            if os.path.isfile(chemin):
                                with open(chemin, "rb") as handle:
                                    donnees_fichier = xmlrpclib.Binary(handle.read())
                                proxy.remote_copie_fichier(app.config['SECRET_KEY'], username, devname_groupe, "data", un_fichier, donnees_fichier)

                    # Appel à l'API de gestion de devoir
                    # reussi = nombre de personnes auquel le devoir a été distribué
                    # rate = liste des personnes n'ayant pas reçu le devoir
                    # dev_name = nom du devoir
                    reussi, rateu, dev_name = proxy.remote_distribute(app.config['SECRET_KEY'], username, groupname, devname_groupe, eleve_only, in_perso)
                    retours[groupname] = {"devoir": dev_name, "réussi": reussi, "rate": rateu}

                    if reussi > 0 and sendmail:
                        envoi_mail(app.config["SERVER_SMTP"], username, groupname, sujetmail, corpsmail)

                # Suppression des dossiers temporaires devoirs et data une fois
                shutil.rmtree(dirtempdev)
                if os.path.isdir(dirtempdata):
                    shutil.rmtree(dirtempdata)

                return make_response(jsonify(retours))
    except Exception, e:
        return make_response(jsonify({"erreur":str(e)}))

@app.route("/changementmdp", methods = ['GET','POST'])
@login_required
def changementmdp():
    """Affichage de la page de changement de mot de passe
    """
    username = get_username()
    proxy = get_proxy()
    est_prof = proxy.remote_est_prof(app.config['SECRET_KEY'], username)

    if not est_prof:
        return render_template('erreurs/non_prof.html', username=username)
    else:

        nb_car_min = app.config['NB_CAR_MIN']
        nb_classe_min = app.config['NB_CLASSE_MIN']

        form = ChangementMDPForm(request.form)
        if request.method == 'POST' and form.validate():
            ancien_mdp = request.form['old_password']
            nouveau_mdp = request.form['password']

            if proxy.remote_change_mdp(app.config['SECRET_KEY'], username, ancien_mdp, nouveau_mdp):
                flash(u'Le mot de passe de "%s" a bien été modifié.' % username)
                return render_template('changementmdp.html', username=username, form=form, nb_car_min=nb_car_min, nb_classe_min=nb_classe_min)
            else:
                flash("Erreur : vérifiez votre ancien mot de passe.")
                return render_template('changementmdp.html', username=username, form=form, nb_car_min=nb_car_min, nb_classe_min=nb_classe_min)
        else:
            return render_template('changementmdp.html', username=username, form=form, nb_car_min=nb_car_min, nb_classe_min=nb_classe_min)

if __name__ == "__main__":
    app.run(host='192.168.122.202', debug=True )
